// Copyright (C) 2011 Oliver Sturm <oliver@oliversturm.com>. All rights reserved.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Permissions;
using System.Runtime.Serialization;
using System.Globalization;

namespace CustomExceptions {
  // Put on this attribute:
  [Serializable]
  public class OliversException : Exception, ISerializable {
    // Standard constructor - perhaps it doesn't always make sense to have this.
    public OliversException( ) { }

    // This is the important constructor to create a new exception of your own type:
    public OliversException(string message, string importantInfo)
      : base(message) {
      this.importantInfo = importantInfo;
    }

    // And a similar one as above, with an inner exception
    public OliversException(string message, string importantInfo, Exception innerException)
      : base(message, innerException) {
      this.importantInfo = importantInfo;
    }

    // This one is for deserialization - should be protected (or even private if
    // your exception class is sealed)
    // Security attribute should be on here to prevent information leaks 
    // with partially trusted code.
    [SecurityPermission(SecurityAction.Demand, SerializationFormatter = true)]
    protected OliversException(SerializationInfo info, StreamingContext context)
      : base(info, context) {
      this.importantInfo = (string) info.GetValue("ImportantInfo", typeof(string));
    }
    
    // Implementing serialization explicitly is important as soon as you have 
    // custom fields in your exception - i.e. most of the time. Sometimes the 
    // standard behavior of the serializers may also be enough - ymmv. 
    // Seems like a clean approach to have this in explicitly for any custom fields.
    // Security attribute should be on here to prevent information leaks 
    // with partially trusted code.
    [SecurityPermission(SecurityAction.Demand, SerializationFormatter = true)]
    public override void GetObjectData(SerializationInfo info, StreamingContext context) {
      base.GetObjectData(info, context);
      info.AddValue("ImportantInfo", ImportantInfo);
    }

    // Include your custom info here. Some MS docs still say you should always implement
    // ToString, but that's not really necessary because it includes the content from 
    // the Message automatically.
    public override string Message {
      get {
        return String.Format(CultureInfo.CurrentCulture, "{0}, ImportantInfo: {1}", base.Message, ImportantInfo);
      }
    }

    // Finally, your own info that makes this exception special
    private string importantInfo;
    public string ImportantInfo {
      get {
        return importantInfo;
      }
    }
  }
}
